<?php
// $Id: contact_importer.admin.inc,v 1.1.2.3 2010/03/24 06:11:34 dalin Exp $

/**
 * @file
 * Admin functions for Contact Importer.
 */

/**
 * Admin info page.
 */
function contact_importer_info() {
  $output = '';

  // Check for requirements.
  $has_errors = _contact_importer_are_requirements_missing();
  $has_info = _contact_importer_are_requirements_missing(FALSE, TRUE);
  $output .= '<h3>'. t('Status') .'</h3>';
  if ($has_errors) {
    $output .= '<p>'. t('Contact Importer has not been setup correctly.  Please visit the '.
      '<a href="@URL">status report</a> to find out more.',
      array('@URL' => url('admin/reports/status'))) .'</p>';
  }
  elseif ($has_info) {
    $output .= '<p>'. t('Contact Importer has been setup correctly but there are additional ways '.
      'to improve it.  Please visit the <a href="@URL">status report</a> to find out more.',
      array('@URL' => url('admin/reports/status'))) .'</p>';
  }
  else {
    $output .= '<p>'. t('Contact Importer has been setup correctly.') .'</p>';
  }

  // Show engines.
  $output .= '<h3>'. t('Enabled Retrieval Engines') .'</h3>';
  $output .= '<p>'. t('Note that some services are fee based.') .'</p>';
  $engines = _contact_importer_engines();
  $providers_enabled = _contact_importer_providers_enabled();
  foreach ($engines as $engine_name => $engine) {
    $output .= '<h4>'. $engine['human_readable'] .'</h4>';
    $output .= '<p>'. l(t('website'), $engine['url']) .' | ' .
      l(t('documentation'), drupal_get_path('module', $engine_name) .'/'. $engine['readme']) .
      '</p>';

    // Show the providers the engine is managing.
    $engine_providers = array();
    foreach ($providers_enabled as $provider_name => $provider) {
      if ($provider['engine_enabled'] == $engine_name) {
        $engine_providers[] = $provider['favicon'] .' '. $provider['human_readable'];
      }
    }
    if (!count($engine_providers)) {
      $output .= '<p>'. t('There are no providers being managed by this engine.') .'</p>';
    }
    else {
      $output .= '<p>'. t('Is managing the providers:') .'</p>';
      $output .= theme('item_list', $engine_providers);
    }
  }
  return $output;
}

/**
 * Admin settings form.
 *
 * @return array
 *  FAPI array.
 */
function contact_importer_settings_form() {
  $form['contact_importer_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Separator'),
    '#description' => t('The character that will be used to separate the list of email addresses: <br />'.
      '%n for new line<br />'.
      '%c for comma separated<br />'.
      'or choose your own.',
      array('%n' => '\n', '%c' => ',')
    ),
    '#default_value' => variable_get('contact_importer_separator', '\n'),
  );
  $form['cache_clear'] = array(
    '#type' => 'fieldset',
    '#title' => 'Clear Contact Importer cached data',
  );
  $form['cache_clear']['cache_clear'] = array(
    '#type' => 'submit',
    '#value' => 'Clear Contact Importer cached data',
    '#submit' => array('contact_importer_cache_clear_all', '_contact_importer_providers_enabled'),
  );
  return system_settings_form($form);
}

/**
 * Admin form to choose what engine to manage each provider.
 *
 * @return array
 *  FAPI array.
 */
function contact_importer_providers_form(&$form_state) {
  $form = array();
  $providers = _contact_importer_providers();
  $providers_enabled = _contact_importer_providers_enabled();
  $form['providers'] = array();
  foreach ($providers as $provider_name => $provider) {
    $provider['engines'] = array('disabled' => 'Disabled') + $provider['engines'];
    $default_value = !empty($provider['engine_enabled']) ? $provider['engine_enabled'] : 'disabled';
    $form['providers'][$provider['type']][$provider_name][$provider_name] = array(
      '#type' => 'select',
      '#title' => $provider['human_readable'],
      '#options' => $provider['engines'],
      '#default_value' => $default_value,
    );

    // Add the domains.
    $domain_list = array();
    foreach ($provider['domains'] as $domain) {
      $domain_list[] = $domain['human_readable'];
    }

    // Split into two columns.
    $domain_count = count($domain_list);
    $half_1 = ceil($domain_count / 2);
    $domain_list_1 = array_slice($domain_list, 0, $half_1);
    $domain_list_2 = array();
    if ($domain_count > 1) {
      $half_2 = $domain_count - $half_1;
      $domain_list_2 = array_slice($domain_list, -$half_2);
    }
    $form['providers'][$provider['type']][$provider_name]['domains'] = array(
      '#value' => theme('item_list', $domain_list_1, '', 'ul', array('style' => 'float:left; font-size:50%')) .
        theme('item_list', $domain_list_2, '', 'ul', array('style' => 'float:left; font-size:50%')),
    );
  }

  if (empty($form['providers'])) {
    drupal_set_message(t('There are no available providers.  Perhaps you have not yet installed '.
      'the appropriate libraries.'), 'error');
  }

  // Make the provider types into fieldsets.
  foreach ($form['providers'] as $provider_type_name => &$provider_type) {
    $provider_type['#type'] = 'fieldset';
    $provider_type['#collapsible'] = TRUE;
    $provider_type['#collapsed'] = TRUE;
    $provider_type['#title'] = t(ucfirst($provider_type_name) . ' providers');
  }
  $form['help'] = array(
    '#value' => '<p>'. t('Choose an engine for the providers that you wish to use on your site.') .'</p>',
    '#weight' => -1,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
    '#suffix' => '<p>'. t('Note that upon saving the form, the icons will be fetched for the '.
      'enabled providers.  This may take a long time depending on the number of enabled providers.'
      ) .'</p>',
  );
  return $form;
}

/**
 * Theme contact_importer_providers_form().
 */
function theme_contact_importer_providers_form($form) {
  $output = drupal_render($form['help']);
  
  // Put each fieldset into a table for easier readability.
  $header = array('Provider', 'Engine', 'Domains');
  foreach (element_children($form['providers']) as $provider_type) {
    $rows = array();
    foreach (element_children($form['providers'][$provider_type]) as $provider_name) {
      $provider =& $form['providers'][$provider_type][$provider_name];
      $title = $provider[$provider_name]['#title'];
      unset($provider[$provider_name]['#title']);
      $row = array(
        $title,
        drupal_render($provider[$provider_name]),
        drupal_render($provider['domains']),
      );
      $rows[] = $row;
    }

    // Put the table into the fieldset.
    $form['providers'][$provider_type]['#value'] = theme('table', $header, $rows);
    $output .= drupal_render($form['providers'][$provider_type]);
  }
  $output .= drupal_render($form);
  return $output;
}

/**
 * Validation handler for contact_importer_providers_form().
 */
function contact_importer_providers_form_validate($form, &$form_state) {

  // Ensure that at least one provider has been enabled.
  $enabled_count = 0;
  $disregarded_keys = array('op', 'submit', 'form_build_id', 'form_token', 'form_id');
  foreach ($form_state['values'] as $provider_name => $engine_name) {
    if (in_array($provider_name, $disregarded_keys)) {
      continue;
    }
    $enabled_count += ($engine_name != 'disabled');
  }
  if (!$enabled_count) {
    form_set_error('help', 'You must enable at least one provider');
  }
  
}

/**
 * Submit handler for contact_importer_providers_form().
 */
function contact_importer_providers_form_submit($form, &$form_state) {
  $disregarded_keys = array('op', 'submit', 'form_build_id', 'form_token', 'form_id');
  $providers_saved = array();
  foreach ($form_state['values'] as $provider_name => $engine_name) {
    if (in_array($provider_name, $disregarded_keys) || $engine_name == 'disabled') {
      continue;
    }
    $providers_saved[$provider_name] = $engine_name;
  }
  variable_set('contact_importer_providers_saved', $providers_saved);
  contact_importer_cache_clear_all();
}

/**
 * Menu callback to display Contact Importer log.
 */
function contact_importer_log() {
  $rows = array();
  $header = array(
    array(
      'data' => t('Log ID'),
      'field' => 'log_id',
      'sort' => 'DESC',
    ),
    array(
      'data' => t('Engine'),
      'field' => 'engine_name',
    ),
    array(
      'data' => t('Provider'),
      'field' => 'provider_name',
    ),
    t('Email'),
    t('Authentication success'),
    t('Contacts Retrieved'),
    t('Contacts Imported'),
    array(
      'data' => t('Completed'),
      'field' => 'completed',
    ),
  );

  $providers = _contact_importer_providers();
  $engines = _contact_importer_engines();

  // Build data.
  $res = pager_query('SELECT * FROM {contact_importer_log} '. tablesort_sql($header), 50);
  while ($log = db_fetch_object($res)) {
    $row = array(
      $log->log_id,
      (isset($engines[$log->engine_name]) ? $engines[$log->engine_name]['human_readable'] : $log->engine_name),
      (isset($providers[$log->provider_name]) ? $providers[$log->provider_name]['human_readable'] : $log->provider_name),
      check_plain($log->email),
      ($log->authentication_success ? 'True' : 'False'),
      $log->contacts_retrieved,
      $log->contacts_imported,
      ($log->completed ? 'True' : 'False'),
    );
    $rows[] = $row;
  }

  // Create output;
  $output = theme('table', $header, $rows);
  $output .= theme('pager', array(), 50);
  return $output;

}